package com.muyu.server.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.CoreDataList;
import com.muyu.common.domain.DataSourceList;
import com.muyu.server.service.CoreDataListService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Author：张腾
 * @Package：com.muyu.server.controller  核心库控制层
 * @Project：cloud-integration
 * @name：DataSourceController
 * @Date：2024/8/20 18:58
 */
@RestController
@RequestMapping("/showDatas")
@Tag(name = "查询数据",description = "从核心库中查询数据")
@AllArgsConstructor
public class CoreDataListController {

    private final CoreDataListService coreDataListService;


    /**
     * 查询核心数据库
     * @param dataSourceList 核心数据库列表请求参数
     * @return 核心数据库列表
     */
    @PostMapping("/selectData")
    @Operation(summary = "查询数据",description = "查询数据")
    public Result<List<CoreDataList>> extractData(
            @Validated @RequestBody DataSourceList dataSourceList
    ){
        return Result.success(coreDataListService.extractData(dataSourceList));
    }



}
