package com.muyu.server.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.CoreDataList;
import com.muyu.common.domain.DataName;
import com.muyu.common.domain.DataValue;
import com.muyu.common.domain.TableNames;
import com.muyu.server.service.DataLinkService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *@Author：yang
 *@Package：com.muyu.server.controller
 *@Project：cloud-property
 *@name：DataLinkController
 *@Date：2024/9/7 20:12
 */
@RestController
@AllArgsConstructor
@RequestMapping("/dataLink")
public class DataLinkController {

    private final DataLinkService dataLinkService;

    /**
     * 查询表数据count
     * @param dataName 数据库名称
     * @param sql sql语句
     * @return 反护试图
     */
    @PostMapping("/findCount")
    public Result<Long> findCount(@RequestParam("dataName") String dataName,@RequestParam("sql") String sql){
        Long longs = dataLinkService.findCount(dataName,sql);
        return Result.success(longs);
    }

    /**
     * 查询表数据dashuju表
     * @param dataName 数据库名称
     * @param sql sql语句
     * @return 反护试图
     */
    @PostMapping("/findSelectDataList")
    public Result<DataValue[][]> findSelectDataList(
            @RequestParam(value = "dataName") String dataName,
            @RequestParam(value = "sql") String sql,
            @RequestParam(value = "dataTableName") String dataTableName,
            @RequestParam(value = "one") Long one,
            @RequestParam(value = "two") Integer two){
        DataValue[][] longs = dataLinkService.findSelectDataList(dataName,sql,dataTableName,one,two);
        return Result.success(longs);
    }

    /**
     * 查询表数据下所有表表
     * @param dataName 数据库名称
     * @return 反护试图
     */
    @PostMapping("/findSelectDataShow")
    public Result<List<DataName>> findSelectDataShow(
            @RequestParam("dataName") String dataName){
        List<DataName> longs = dataLinkService.findSelectDataShow(dataName);
        return Result.success(longs);
    }

    /**
     * 查询表数据dashuju表
     * @param dataTableName 查看表下的所有数据
     * @return 反护试图
     */
    @PostMapping("/findSelectTableShow")
    public Result<List<TableNames>> findSelectTableShow(
            @RequestParam(value = "dataTableName") String dataTableName){
        List<TableNames> longs = dataLinkService.findSelectTableShow(dataTableName);
        return Result.success(longs);
    }

//    /**
//     * 查询表数据dashuju表
//     * @param dataTableName 查看表下的所有数据
//     * @return 反护试图
//     */
//    @PostMapping("/AddInsertTableShow")
//    public Result<List<TableNames>> AddInsertTableShow(
//            @RequestParam("dataTableName") String dataTableName){
//        List<TableNames> longs = dataLinkService.AddInsertTableShow(dataTableName);
//        return Result.success(longs);
//    }

}
