package com.muyu.server.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.*;
import com.muyu.server.service.DataSourceService;
import com.muyu.server.service.TableFieldService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author：张腾
 * @Package：com.muyu.server.controller 抽取数据结构列表
 * @Project：cloud-integration
 * @name：DataSourceController
 * @Date：2024/8/20 18:58
 */
@RestController
@RequestMapping("/dataSource")
@Tag(name = "抽取数据",description = "从数据源抽取数据")
@AllArgsConstructor
public class DataSourceController {

    private final DataSourceService dataSourceService;


    /**
     * 抽取数据列表
     * @param connect 数据实体类
     * @return 返回试图
     */
    @PostMapping("/extractData")
    @Operation(summary = "抽取数据",description = "从数据源中抽取数据")
    public Result<List<DataSourceList>> extractData(@RequestBody Connect connect){
        return Result.success(dataSourceService.extractData(connect),"您的数据已成功获取");
    }

    /**
     * 抽取结构
     * @param connect 实体类
     * @return 返回试图
     */
    @PostMapping("/tableField")
    @Operation(summary = "抽取结构",description = "从数据源中抽取结构")
    public Result<List<TableFie>> extractTableField(@RequestBody Connect connect){
        List<TableFie> tableFIES = dataSourceService.extractTableField(connect);
        if (CollectionUtils.isEmpty(tableFIES)){
            return Result.success(null,"未查询到结构，请检查数据来源");
        }
        return Result.success(tableFIES,"您的数据已成功获取");
    }




//
//    /**
//     * 获取数据库表名
//     * @return 返回试图
//     */
//    @PostMapping("/tableName")
//    @Operation(summary = "获取数据库表名",description = "从数据库中抽取数据库表名称")
//    public Result<List<TableNames>> extractTableName(){
//        List<TableNames> tableFIES = dataSourceService.extractTableName();
//        if (StringUtils.isEmpty(tableFIES)){
//            return Result.success(null,"未查询到结构，请检查数据来源");
//        }
//        return Result.success(tableFIES,"您的数据已成功获取");
//    }

}
