package com.muyu.server.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.TableFie;
import com.muyu.common.domain.TableNames;
import com.muyu.server.service.TableFieldService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author：yang
 * @Package：com.muyu.server.controller 数据字段列表控制层
 * @Project：cloud-property
 * @name：TableFieldController
 * @Date：2024/8/23 19:56
 */
@RestController
@RequestMapping("/tableField")
@Tag(name = "抽取数据字段",description = "从数据源抽取数据字段")
@AllArgsConstructor
public class TableFieldController {

    private final TableFieldService tableFieldService;

    /**
     * 查询所有数据字段
     * @ 所有数据字段列表请求参数
     * @return 数据字段列表
     */
    @RequestMapping(path = "/list" ,method = RequestMethod.POST)
    @Operation(summary = "抽取数据字段",description = "所有数据字段列表请求参数")
    public Result<List<TableFie>> selectList(){
        return Result.success(
                tableFieldService.selectList(),"已同步"
        );
    }

    /**
     * 添加方法
     * @param tableFie
     * @return
     */
    @PostMapping("/add")
    @Operation(summary = "数据字段添加",description = "添加数据字段信息，添加成功之后才可以使用数据字段")
    public Result<String> save(@Validated @RequestBody TableFie tableFie){
        tableFieldService.save(TableFie.addBuild(tableFie));
        return Result.success(null,"操作成功");
    }

    /**
     * 修改方法
     * @param tableFie 修改方法请求信息
     * @return 修改结果
     */
    @PutMapping("/update/{fieId}")
    @Operation(summary = "数据字段修改",description = "通过ID修改数据字段信息")
    public Result<String> update(
            @Schema(title = "数据字段ID",type = "Long",defaultValue = "1",description = "修改数据字段信息需要依据的唯一条件")
            @PathVariable("fieId") Long fieId,
            @RequestBody @Validated TableFie tableFie){
        tableFieldService.updateById(TableFie.updBuild(tableFie,()-> String.valueOf(fieId)));
        return Result.success(null,"操作成功");
    }

    /**
     * 删除客户
     * @param orderCustomerId 删除客户请求信息
     * @return 删除结果
     */
    @DeleteMapping("/delete/{fieId}")
    @Operation(summary = "客户信息删除",description = "通过ID删除客户信息")
    public Result<String> delete(@PathVariable("fieId") Long orderCustomerId){
        tableFieldService.removeById(orderCustomerId);
        return Result.success(null,"操作成功");
    }

    /**
     * 功能:根据表名查询所有字段信息
     *
     * @param tableName 表名
     * @return 字段集合
     */
    @PostMapping("/selectByTableName")
    @Operation(summary = "根据表名获取所有字段")
    public Result<List<TableFie>> selectByName(@RequestParam("tableName") String tableName){
        return Result.success(tableFieldService.selectByName(tableName),"操作成功");
    }
}
