package com.muyu.server.service.impl;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.muyu.common.datascope.annotation.DataScope;
import com.muyu.common.domain.*;
import com.muyu.common.domain.enums.DataType;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.LoginUser;
import com.muyu.common.system.domain.SysDept;
import com.muyu.common.system.domain.SysUser;
import com.muyu.server.mysql.MySqlQuery;
import com.muyu.server.mapper.DataRunNameMapper;
import com.muyu.server.service.*;
import com.muyu.server.util.JdbcHelper;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.sql.*;
import java.util.*;

/**
 * @Author：yang
 * @Package：com.muyu.server.service.impl
 * @Project：cloud-property
 * @name：DataRunNameImpl
 * @Date：2024/8/27 20:01
 */
@Service
@Log4j2
public class DataRunNameServiceImpl implements DataRunNameService {

    @Autowired
    private TableRunNameService tableRunNameService;
    @Autowired
    private DataNameService dataNameService;
    @Autowired
    private DataSourceService dataSourceService;
    @Autowired
    private TableFieldService tableFieldService;
    @Autowired
    private DataRunNameMapper dataRunNameMapper;

    /**
     * 添加 数据库名，表名，结构
     */
    @Override
    public String addDataTableName(Connect connect) {

        //获取数据库名称
        DataName dataName1 = new DataName();
        dataName1.setName(connect.getDatabaseName());
        dataNameService.save(dataName1);

        //获取数据库表的名称
        List<TableNames> tableNames = tableRunNameService.extractTableName(connect.getDatabaseName());
        tableNames.forEach(tableName -> {
            log.info("数据库ID:{}",dataName1.getId());
            tableName.setDataId(dataName1.getId());
        });

        tableRunNameService.saveBatch(tableNames);
        if (CollectionUtils.isEmpty(tableNames))
        {
            return "数据库里不存在表";
        }
//        List<TableFie> tableFies = null;
//
        //遍历数据表名称
        for (TableNames tableName : tableNames) {

            log.info("数据库表ID:{}",tableName.getId());
            connect.setTableName(tableName.getName());
            List<TableFie> tableFies = dataSourceService.extractTableField(connect);
            tableFies.forEach(tableFy -> {
                tableFy.setTableId(tableName.getId());
            });
            if (CollectionUtils.isEmpty(tableFies))
            {
                return "数据库里表没有字段";
            }
            log.info("开始添加表结构与数据:{}",tableFies);
            tableFieldService.saveBatch(tableFies);
        }

        return "添加成功";
    }

    /**
     * 查询数据库名称 和 表名称联查
     * @return
     */
    @Override
    public List<DataName> getDatabasesName() {

        List<DataName> dataNames = dataNameService.extractDataName();
        dataNames.forEach(dataName -> {
            String name = dataName.getName();
            List<TableNames> tableNames = tableRunNameService.extractTableName(name);
            dataName.setTableNames(tableNames);
        });
        return dataNames;
    }

    /**
     * 资产展示
     *  连接数据库获取当前所有的数据库名称和所有字段
     * @return 所有数据库以及所有表的结果集
     */
    @Override
    public List<DataName> extractDataTableNameShow() {
        //数据库名称查询
        List<DataName> dataNameList = dataNameService.list().stream()
                .map(DataName::dataNameBuild).toList();

        //遍历获取数据数据库名称对象
        dataNameList.forEach(dataName -> {
            HashSet<TableNames> set = new HashSet<>();
            //判断getId不为空
            if (StringUtils.isNotBlank(String.valueOf(dataName.getId()))) {

                //使用安全管理工具获取当前登录的用户对象
                LoginUser loginUser = SecurityUtils.getLoginUser();
                //从登录用户对象中获取系统用户对象
                SysUser sysUser = loginUser.getSysUser();
                //从系统用户对象中获取用户ID
                Long userId = sysUser.getUserId();
                //从系统用户对象中获取部门ID
                Long deptId = sysUser.getDeptId();
                // 调用DataRunNameServiceImpl服务的findTableIdAndBasicIdByUserId方法
                // 传入用户ID，
                // 获取该用户授权的所有表ID和基础ID
                List<AssetAuthorization> tableIdAndBasicIdByUserId = dataRunNameMapper.findTableIdAndBasicIdByUserId(userId);

                // 调用assetImpowerService服务的findTableIdAndBasicIdByDeptId方法
                // 传入部门ID，获取该部门下所有授权的表ID和基础ID
                List<AssetAuthorization> idByDeptId = dataRunNameMapper.findTableIdAndBasicIdByDeptId(deptId);
                //在取数据数据遍历里获取表名称查询方法
                tableIdAndBasicIdByUserId.forEach(tableIdAndBasicId -> {
                    List<TableNames> tableNames = this.dataDeptShow(tableIdAndBasicId, dataName);
                    System.out.println(tableNames);
                    tableNames.forEach(tableName -> {
                        set.add(tableName);
                    });

                });
                idByDeptId.forEach(tableIdAndBasicId -> {
                        List<TableNames> tableNames = this.dataDeptShow(tableIdAndBasicId, dataName);
                    System.out.println(tableNames);
                        tableNames.forEach(tableName -> {
                            set.add(tableName);
                        });
                });
            }
            List<TableNames> tableNameSet = new ArrayList<>(set);
            dataName.setTableNames(tableNameSet);
        });
        //返回所有数据
        return dataNameList;
    }

    private  List<TableNames> dataDeptShow(AssetAuthorization tableIdAndBasicId, DataName dataName) {
        List<TableNames> tableNamesList = tableRunNameService.list(new LambdaQueryWrapper<TableNames>()
                        .eq(TableNames::getDataId, dataName.getId())
                        .eq(TableNames::getId, tableIdAndBasicId.getTableId()))

                .stream()
                .map(TableNames::tableNamesBuild)
                .toList();

        //判断tableNamesList查询数据为空直接返回
        if (CollectionUtils.isEmpty(tableNamesList)) {
            return null;
        }

        //在每一次循环中创建一个新的集合
        ArrayList<TableNames> namesArrayList = new ArrayList<>(tableNamesList);
        //添加到数据库实体类集合字段上
        dataName.setTableNames(namesArrayList);
        //将添加好的实体类集合进行遍历操作
        log.info("数据库表名称:{}",dataName.getTableNames());
        if (CollectionUtils.isEmpty(dataName.getTableNames())) {return null;}
        //在内部封装方法
        dataName.getTableNames().forEach(tableName1 -> {
            List<TableFie> tableFies = this.tableStructureShow(tableName1);
            tableName1.setTableFie(tableFies);
        });
        return tableNamesList;
    }

    /**
     *  连接数据库获取当前所有的数据库名称和所有字段
     * @return 所有数据库以及所有表的结果集
     */
    @Override
    public List<DataName> getDataBaseTableName() {

        //数据库名称查询
        List<DataName> dataNameList = dataNameService.list().stream()
                .map(DataName::dataNameBuild).toList();

        //遍历获取数据数据库名称对象
        dataNameList.forEach(dataName -> {

            //判断getId不为空
            if (StringUtils.isNotBlank(String.valueOf(dataName.getId()))) {

                //在取数据数据遍历里获取表名称查询方法
                List<TableNames> tableNamesList = tableRunNameService.list(new LambdaQueryWrapper<TableNames>()
                                .eq(TableNames::getDataId, dataName.getId()))
                                .stream()
                                .map(TableNames::tableNamesBuild)
                                .toList();

                //判断tableNamesList查询数据为空直接返回
                if (CollectionUtils.isEmpty(tableNamesList)) {
                    return;
                }

                //在每一次循环中创建一个新的集合
                ArrayList<TableNames> namesArrayList = new ArrayList<>(tableNamesList);
                //添加到数据库实体类集合字段上
                dataName.setTableNames(namesArrayList);
                //将添加好的实体类集合进行遍历操作
                log.info("数据库表名称:{}",dataName.getTableNames());
                if (CollectionUtils.isEmpty(dataName.getTableNames())) {return;}
            }
        });
        //返回所有数据
        return dataNameList;

    }

    /**
     * 查询值
     *
     * @param dataDame
     * @param tableName
     * @param field
     * @return
     */
    private String tableFySelect(String dataDame, String tableName, String field) {

        // 从连接池获取数据库连接
        DruidDataSource conn = JdbcHelper.getConnRs(dataDame);
        // 初始化一个列表，用于存储数据值对象
        try {
            DruidPooledConnection connection = conn.getConnection();
            // 准备SQL查询语句

            PreparedStatement ps = connection.prepareStatement("select " + field + " FROM " + tableName+" LIMIT 1");
            ResultSet resultSet = ps.executeQuery();
            while (resultSet.next()) {
                String fields = resultSet.getString(field);
                // 返回包含数据值的列表
                return fields;
            }

            // 释放数据库连接
        } catch (SQLException e) {
            // 如果发生SQL异常，抛出运行时异常
            throw new RuntimeException(e);
        }
        return null;
    }

    /**
     * 数据类型转换
     * @para 数据类型转换ID
     * @return 返回结果
     */
    @Override
    public List<DataValue> findTableValue( String dataId,String tableName) {
        DataName byId = dataNameService.getById(dataId);
        MySqlQuery mySqlQuery = new MySqlQuery();
        // 设置数据源ID
        mySqlQuery.setDataSourceId(String.valueOf(dataId));

        DruidDataSource connRs1 = JdbcHelper.getConnRs(byId.getName());
        DruidPooledConnection conn = null;
        try {
            conn = connRs1.getConnection();

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

        List<DataValue> list = new ArrayList<>();

        try {
            PreparedStatement preparedStatement = conn.prepareStatement("select * FROM "+tableName +" LIMIT 1");

            this.getDataResult(conn, list, preparedStatement);
            JdbcHelper.close(conn,null,null);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return list;
    }

    /**
     * 授权后资产展示
     * @param conn
     * @param list
     * @param preparedStatement
     * @throws SQLException
     */
    private void getDataResult(DruidPooledConnection conn, List<DataValue> list, PreparedStatement preparedStatement) {
       try {
           ResultSet resultSet = preparedStatement.executeQuery();
           ResultSetMetaData metaData = resultSet.getMetaData();
           int columnCount = metaData.getColumnCount();

           HashMap<Integer, DataValue> map = new HashMap<>();
           while (resultSet.next()){
               for (int i = 1; i <= columnCount; i++) {

                   if (resultSet.isFirst()){
                       String columnTypeName = metaData.getColumnTypeName(i);

                       DatabaseMetaData metaData1 = conn.getMetaData();
                       ResultSet columns = metaData1.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                       String remarks =null;
                       while (columns.next()){
                           remarks = columns.getString("REMARKS");
                           log.info("字段备注:{}", remarks);
                       }
                       DataValue build = DataValue.builder()
                               .key(metaData.getColumnName(i))
                               .label(remarks)
                               .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                               .type(DataType.findBySqlType(columnTypeName))
                               .build();
                       list.add(build);
                       map.put(i, build);
                   }else {
                       DataValue build = DataValue.builder()
                               .key(metaData.getColumnName(i))
                               .label(map.get(i).getLabel())
                               .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                               .type(map.get(i).getType())
                               .build();
                       list.add(build);
                   }

               }
           }
           JdbcHelper.close(null,preparedStatement,null);
       } catch (Exception e) {
           log.error(e.getMessage(),e);
       }
    }

    /**
     * 数据类型转换
     * @par 数据类型转换ID
     * @return 返回结果
     */
    @Override
    public List<DataValue> selectBasicList() {
        return List.of();
    }

    /**
     * 获取表中的数据值前台调用 闪闪
     *
     * @param basicId   响应参数
     * @param tableName 响应参数
     * @param dataId
     * @return 返回结果
     */
    @Override
    public List<DataValue> findTableValueByTableName(Long basicId, String dataId, String tableName) {
        {
            // 创建一个MySQL查询对象
            MySqlQuery mySqlQuery = new MySqlQuery();
            // 设置数据源ID
            mySqlQuery.setDataSourceId(String.valueOf(basicId));
            // 从服务中获取TableFie对象，根据基本ID
            TableFie byId = tableFieldService.getById(basicId);
            // 创建MySQL连接池对象，并初始化
            DataName byIds = dataNameService.getById(dataId);


            // 从连接池获取数据库连接
            DruidDataSource conn = JdbcHelper.getConnRs(byIds.getName());
            // 初始化一个列表，用于存储数据值对象
            List<DataValue> list = new ArrayList<>();
            PreparedStatement preparedStatement = null;
            try {
                DruidPooledConnection connection = conn.getConnection();
                // 准备SQL查询语句
                connection.prepareStatement("select * FROM "+tableName);
                // 执行查询，获取结果集
                getDataResult(connection, list, preparedStatement);
                // 释放数据库连接

            } catch (SQLException e) {
                // 如果发生SQL异常，抛出运行时异常
                throw new RuntimeException(e);
            }

            // 返回包含数据值的列表
            return list;
        }
    }

    /**
     * 查询部门管理数据
     *
     * @param dept 部门信息
     *
     * @return 部门信息集合
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysDept> selectDeptList (SysDept dept) {
        return dataRunNameMapper.selectDeptList(dept);
    }

    /**
     * 添加用户权限的信息
     * @param authorization 相应参数
     * @return 返回结果
     */
    @Override
    public boolean addUserAssetAuthorization(AssetAuthorization authorization) {
        Integer add = dataRunNameMapper.addUserAssetAuthorization(authorization);
        if (add>0){
            return true;
        }
        return false;
    }

    /**
     * 添加部门权限的信息
     * @param authorization 响应参数
     * @return 返回结果
     */
    @Override
    public boolean addDeptAssetAuthorization(AssetAuthorization authorization) {
        Integer add = dataRunNameMapper.addDeptAssetAuthorization(authorization);
        if (add>0){
            return true;
        }
        return false;
    }

    @Override
    public List<TableFie> tableStructureShow(TableNames tableNames) {

        List<TableFie> tableFieList = null;
        if (StringUtils.isNotBlank(String.valueOf(tableNames.getId()))) {
            tableFieList = tableFieldService.list(new LambdaQueryWrapper<TableFie>()
                            .eq(TableFie::getTableId, tableNames.getId()))
                    .stream()
                    .map(TableFie::tableFieBuild).toList();
            log.info("字段类型名称集合：{}", tableFieList);
            DataName dataName = dataNameService.getById(tableNames.getDataId());

            List<DataValue> tableValue = this.findTableValue(String.valueOf(dataName.getId()), tableNames.getName());

            tableFieList.forEach(tableFy -> {
                //查询字段映射信息
                String str = this.tableFySelect(dataName.getName(), tableNames.getName(), tableFy.getField());
                tableFy.setDataValue(str);

                log.info("字段数据对象:{}", tableFy);

                for (DataValue dataValue : tableValue) {
                    if (tableFy.getField().equals(dataValue.getKey())) {
                        tableFy.setDataMapping(dataValue.getType().getJavaType());
                    }
                }
            });
            log.info("字段数据信息:{}", tableFieList);
        }
        return tableFieList;
    }
}
